#ifndef ENCODER_H_
#define ENCODER_H_
/****************************************************************************
 Title	:   Drehencoder Library
 Author:    Frank Klee
 Date:	    2014/12/04
 Target:    ATMega8

 DESCRIPTION
		Basisroutinen zum Abfragen eines Drehencoders mittels Polling.
		
		Bei Bedarf wird Timer0 fr das Polling verwendet.
       
*****************************************************************************/

#ifndef F_CPU
#define F_CPU 1000000
#endif

//Art des Drehencoders definieren
//#define Single_Step
//#define Two_Step
#define Four_Step

#define ENC_A_PORT      PORTC        /**< port for line A         */
#define ENC_A_PIN       1            /**< pin  for line A         */
#define ENC_B_PORT      PORTC        /**< port for line B         */
#define ENC_B_PIN       0            /**< pin  for line B         */
#define ENC_T_PORT      PORTC        /**< port for button         */
#define ENC_T_PIN       2            /**< pin  for button         */

//Timer 0 wird zum Abfragen des Encoders genutzt. Falls das strt, Parameter auf 0 setzen
//und die Polling-Routine alle 1-2ms vom Hauptprogramm aus aufrufen.
#define Use_T0_Interrupt 1

typedef enum EButtonPressedState
{
	ButtonPressed_Unpressed,
	ButtonPressed_Short,
	ButtonPressed_Long
	
}tEButtonPressedState;
//Initialisiert den Encoder und aktiviert den Interrupt + Timer
extern void encoder_init( void );  
//Liest die Position des Encoders aus
//Wenn Ueberlauf=1 dann zhlt der Encoder nach Max
//wieder von Min und umgekehrt
extern int8_t encoder_read( char Ueberlauf );
//Ruft den Status des Encoder-Knopfes
extern tEButtonPressedState encoder_GetButtonState();
//Setzt die aktuelle Drehencoderposition
extern void encoder_write(int8_t EncoderPos);
//Setzt Min- und Max-Werte fr die Drehgeberposition
extern void encoder_MinMax(int8_t EncoderMin,int8_t EncoderMax);
#if (Use_T0_Interrupt==0)
//Fragt den Encoder ab
//Mu alle 1-2ms aufgerufen werden
extern void encoder_polling(void);
#endif
#endif /* ENCODER_H_ */
