/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * main.c - demo main module to test irmp decoder
 *
 * Copyright (c) 2009-2010 Frank Meyer - frank(at)fli4l.de
 *
 * ATMEGA88 @ 8 MHz
 *
 * Fuses: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */


#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include "irmp.h"
#include "irsnd.h"
#include "lcd.h"
#include "encoder.h"
#include "uart.h"

#define LCD_LIGHT_DDR  DDRC
#define LCD_LIGHT_PORT PORTC
#define LCD_LIGHT 5

/* define pin for enter-self-prog-mode */
#define	PROGPORT	PORTC
#define	PROGPIN	    PINC
#define	PROG_NO	    PC4


#define UART_BAUD_RATE 38400			// Baudrate fr serielle Schnittstelle
#define LCD_LIGHTTIME 200000   // = 10s Light if signal received

#ifndef F_CPU
#error F_CPU unkown
#endif


volatile uint32_t lcd_lighttimer=0;
volatile unsigned char lcd_lightperm=0;
volatile unsigned char command_buffer[20];
volatile unsigned char command_complete;

void
timer_init (void)
{
  OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                    // compare value: 1/20000 of CPU frequency
  TCCR1B  = (1 << WGM12) | (1 << CS10);                                     // switch CTC Mode on, set prescaler to 1

#if defined (__AVR_ATmega8__) || defined (__AVR_ATmega16__) || defined (__AVR_ATmega32__) || defined (__AVR_ATmega64__) || defined (__AVR_ATmega162__) 
  TIMSK  = 1 << OCIE1A;                                                     // OCIE1A: Interrupt by timer compare
#else
  TIMSK1  = 1 << OCIE1A;                                                    // OCIE1A: Interrupt by timer compare
#endif	// __AVR...
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/20000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// Timer 1 output compare A interrupt service routine
ISR(TIMER1_COMPA_vect)
{
	static uint16_t command_timer;
	static unsigned char command_buffer_vect;
	unsigned int c;
	
	if (! irsnd_ISR())          // call irsnd ISR
	{                           // if not busy...
		irmp_ISR();             // call irmp ISR
	}
//	irmp_ISR();                     // call irmp ISR
//	irsnd_ISR();					// call irsnd ISR
	// call other timer interrupt routines...

	if (!lcd_lightperm)
	{
		// switch off Light after Signal is decodet
		if(lcd_lighttimer)lcd_lighttimer--;
		else 
		{
			LCD_LIGHT_PORT &= ~(1<< LCD_LIGHT);
		}
	}

	if (command_timer)command_timer--;
	else
	{
		//alle 100ms empfangene Zeichen in Befehlspuffer schreiben
		command_timer=2000;
		if (!command_complete)		//wenn Befehlspuffer noch voll
		{
			c=uart_getc();
			while (!(c & UART_NO_DATA))
			{
				command_buffer[command_buffer_vect]=c;
				if (command_buffer[command_buffer_vect]==0x0d)		//Befehlsabschlu mit CR
				{
					command_buffer[command_buffer_vect]=0x00;		//Ende des Strings
					command_buffer_vect=0;
					command_complete=1;
				}
				else
				{
					++command_buffer_vect;
				}
				c=uart_getc();		//nchstes Zeichen lesen
			}			
		}
	}
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MAIN: main routine
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
int
main (void)
{
  IRMP_DATA irmp_data;
  char s[10];
    void (*bootptr)( void ) = 0x7800; // Set up function pointer to BOOTLOADER.
	unsigned char temp;
	unsigned char temp1;
	char Bef[10];
	char Code[10];
	char Address[10];
	char Command[10];
	char Flags[10];
	char Button;
  
    PROGPORT |= (1<<PROG_NO); // Enable pull-up on PROG_NO line on PROGPORT.

  uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));

  irmp_init();                                                              // initialize irmp
  irsnd_init();																// initialize irsnd
  timer_init();                                                             // initialize timer
  encoder_init();															// Drehencoder initialisieren
  sei ();                                                                   // enable interrupts

  uart_putc(0x0d);
  uart_putc(0x0a);
  uart_puts("IRMP-LCD V2.0 www.mikrocontroller-projekte.de");
  uart_putc(0x0d);
  uart_putc(0x0a);
  #if IRMP_LOGGING == 1
  uart_puts("Logging Mode");
  uart_putc(0x0d);
  uart_putc(0x0a);
  #endif

  /* initialize display, cursor off */
  lcd_init(LCD_DISP_ON);
  /* clear display and home cursor */
  lcd_clrscr();
     
  /* put light to Display if available */
  LCD_LIGHT_PORT |= (1<<LCD_LIGHT);   // switch light on
  lcd_lighttimer=LCD_LIGHTTIME;  // automatically switch of after LIGHTTIME

  /* put string to display (line 1) with linefeed */
  lcd_puts("IRMP LCD V2.0");

  for (;;)
  {
    if( !(PROGPIN & (1<<PROG_NO)) ) // If PROGPIN is pulled low, enter programmingmode.
	{
		/* clear display and home cursor */
		lcd_clrscr();
		/* put string to display (line 1) with linefeed */
		lcd_puts("BootloaderStart");
		while ( !(PROGPIN & (1<<PROG_NO)));	//Wait for PROGPIN is high (button released)
		lcd_gotoxy(0,1);
		lcd_puts("Knopf drcken");
		temp = encoder_GetButtonState();
		while (temp==ButtonPressed_Unpressed)
		{
			temp = encoder_GetButtonState();
		}
		if (temp==ButtonPressed_Short)
		{
			lcd_clrscr();
			lcd_puts("Bootloader");
			lcd_gotoxy(0,1);
			lcd_puts("started");
			bootptr();	
		}
		lcd_clrscr();
	}	
    if (irmp_get_data (&irmp_data))
    {
        // ir signal decoded, do something here...
        // irmp_data.protocol is the protocol, see irmp.h
        // irmp_data.address is the address/manufacturer code of ir sender
        // irmp_data.command is the command code
 
#if IRMP_LOGGING != 1

        lcd_clrscr();
		lcd_puts(itoa(irmp_data.protocol,s,16));
		lcd_puts("/");
		lcd_puts_p(pgm_read_word(&(irmp_protocol_names[irmp_data.protocol])));
		lcd_gotoxy(0,1);
        lcd_puts("A:");
		lcd_puts(itoa(irmp_data.address,s,16));
        lcd_puts(" C:");
		lcd_puts(itoa(irmp_data.command,s,16));
		lcd_puts(" R");
		lcd_puts(itoa(irmp_data.flags,s,16));

        uart_puts("R: Code: ");
		uart_puts(itoa(irmp_data.protocol,s,16));
		uart_puts("/");
		uart_puts_p (pgm_read_word (&(irmp_protocol_names[irmp_data.protocol])));
        uart_puts("    A: ");
		uart_puts(itoa(irmp_data.address,s,16));
        uart_puts("    C: ");
		uart_puts(itoa(irmp_data.command,s,16));
        uart_puts("    R: ");
		uart_puts(itoa(irmp_data.flags,s,16));
		uart_puts("\n");
        /* put light to Display if available */
        LCD_LIGHT_PORT |= (1<<LCD_LIGHT);   // switch light on
        lcd_lighttimer=LCD_LIGHTTIME;  // automatically switch of after LIGHTTIME
#endif
    }
/*
** Befehl ber serielle Schnittstelle interpretieren
** empfangener Befehl steht komplett im Befehlspuffer
*/
	if (command_complete)
	{
		unsigned char command_correct=0;
		switch(command_buffer[0])
		{
			case '?':
				uart_puts("IR-Tester V2.0\n");
				command_correct=1;
				break;
			case 'v':
				uart_puts("Version IRMP:  2.8.2 07.05.2015\n");
				uart_puts("Version IRSND: 2.8.2 07.05.2015\n");
				command_correct=1;
				break;
			case 'c':
				for (temp=1;temp<=IRMP_N_PROTOCOLS;temp++)
				{
					uart_puts(itoa(temp,s,16));
					uart_puts(";");
					uart_puts_p (pgm_read_word (&(irmp_protocol_names[temp])));
					uart_puts("\n");
				}
				command_correct=1;
				break;
			case 's':
				//nachsehen, ob gengend Parameter
				temp=0;
				temp1=0;
				while (command_buffer[temp])
				{
					if (command_buffer[temp]==';') temp1++;
					temp++;
				}
				if (!(temp1==4))
				{
					//Anzahl der Parameter <>4; Formatfehler
					command_correct=2;
					break;
				}
				irmp_data.flags = 0;					// don't repeat frame
				char *Bef=strtok(command_buffer,";");
				char *Code=strtok(NULL,";");
				char *Address=strtok(NULL,";");
				char *Command=strtok(NULL,";");
				char *Flags=strtok(NULL,";");
				irmp_data.protocol=strtol(Code,NULL,16);
				irmp_data.address=strtol(Address,NULL,16);
				irmp_data.command=strtol(Command,NULL,16);
				irmp_data.flags=strtol(Flags,NULL,10);
				if (irmp_data.protocol>IRMP_N_PROTOCOLS)
				{
					command_correct=4;
					break;
				}
				if (irsnd_send_data (&irmp_data, TRUE)) 		// send frame, wait for completion
				{
					command_correct=1;
				}
				else
				{
					command_correct=3;
				}
				break;
			case 'h':
				uart_puts("folgende Befehle sind moeglich:\n");
				uart_puts("?                                 Geraetename und Softwareversion\n");
				uart_puts("v                                 Versionen von IRMP und IRSND\n");
				uart_puts("c                                 alle IR-Codes und -Codenamen ausgeben\n");
				uart_puts("s;[Code];[Adresse];[Befehl];[Wiederholungen<=14]       IR-Befehl senden\n");
				uart_puts("h                                 diese Liste\n");
				command_correct=1;
		}		
		command_complete=0;			//Flag zurcksetzen, damit weitere Befehle gelesen werden knnen
		switch (command_correct)
		{
			case 1:
				uart_puts("OK\n");		//OK senden, da Befehl abgearbeitet
				break;
			case 2:
				uart_puts("falsche Anzahl Parameter\n");
				break;
			case 3:
				uart_puts("Fehler beim Senden\n");
				break;
			case 4:
				uart_puts("Maximalwert fuer Code = ");
				uart_puts(itoa(IRMP_N_PROTOCOLS,s,16));
				uart_puts("\n");
				break;
			default:
				uart_puts("ungueltiger Befehl\n");
		}
	}
/*
** Prfen, ob Taster des Drehimpulsgebers gedrckt wurde
** wenn ja, Daten fr das Senden eines Frame zusammenstellen
*/
	Button=encoder_GetButtonState();
	if (Button==ButtonPressed_Short)
	{
		LCD_LIGHT_PORT |= (1<<LCD_LIGHT);   // switch light on
		lcd_lighttimer=LCD_LIGHTTIME;
		lcd_lightperm=1;
		
		//Daten des letzten empfangenen Telegramms anzeigen
		//Cursor auf Protokoll
		lcd_clrscr();
		lcd_puts("<");
		lcd_puts(itoa(irmp_data.protocol,s,16));
		lcd_puts("/");
		lcd_puts_p(pgm_read_word(&(irmp_protocol_names[irmp_data.protocol])));
		lcd_puts(">");
		lcd_gotoxy(0,1);
		sprintf(s,"A:%4x",irmp_data.address);
		lcd_puts(s);
		sprintf(s," C:%4x",irmp_data.command);
        lcd_puts(s);
		irmp_data.flags=0;		//keine Wiederholung
		lcd_puts(" R0");
		encoder_MinMax(1,IRMP_N_PROTOCOLS);
		encoder_write(irmp_data.protocol);
		while(encoder_GetButtonState()==ButtonPressed_Unpressed)
		{
			irmp_data.protocol=encoder_read(0);
			lcd_gotoxy(1,0);
			lcd_puts("               ");
			lcd_gotoxy(1,0);
			lcd_puts(itoa(irmp_data.protocol,s,16));
			lcd_puts("/");
			lcd_puts_p(pgm_read_word(&(irmp_protocol_names[irmp_data.protocol])));
			lcd_puts(">");
			_delay_ms(50);
		}		
		Button = encoder_GetButtonState();
		if (Button!=ButtonPressed_Long)		//wenn Button lange gedrckt, sofort aussteigen und senden
		{
			lcd_gotoxy(0,0);
			lcd_puts(" ");
			lcd_puts(itoa(irmp_data.protocol,s,16));
			lcd_puts("/");
			lcd_puts_p(pgm_read_word(&(irmp_protocol_names[irmp_data.protocol])));
			lcd_puts(" ");
			
			//Adresse eingeben
			//Cursor auf erste Ziffer der Adresse
			lcd_gotoxy(2,1);
			lcd_command(LCD_DISP_ON_CURSOR_BLINK);
			encoder_MinMax(0,15);
			temp=irmp_data.address>>12;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if (temp==0)
				{
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(2,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			if (temp>0)
			{
				//hchstwertige Stelle <>0
				temp1=1;
			}
			else
			{
				temp1=0;
			}
			irmp_data.address&=0x0fff;
			irmp_data.address|=(temp<<12);
			
			//zweite Ziffer der Adresse
			lcd_gotoxy(3,1);
			encoder_MinMax(0,15);
			temp=(irmp_data.address>>8)&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if ((temp==0) & (temp1==0))
				{
					//hherwertige Stelle ist auch 0
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(3,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			if (temp>0) temp1=1;	//wenn aktuelle Stelle >0, dann an den weiteren Stellen 0 statt Blank ausgeben
			irmp_data.address&=0xf0ff;
			irmp_data.address|=(temp<<8);
			
			//dritte Ziffer der Adresse
			lcd_gotoxy(4,1);
			encoder_MinMax(0,15);
			temp=(irmp_data.address>>4)&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if ((temp==0)&(temp1==0))
				{
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(4,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			irmp_data.address&=0xff0f;
			irmp_data.address|=(temp<<4);
			
			//vierte Ziffer der Adresse
			lcd_gotoxy(5,1);
			encoder_MinMax(0,15);
			temp=irmp_data.address&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				lcd_puts(itoa(temp,s,16));
				lcd_gotoxy(5,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			irmp_data.address&=0xfff0;
			irmp_data.address|=temp;
			

			//Befehl eingeben
			//Cursor auf erste Ziffer des Befehls
			lcd_gotoxy(9,1);
			encoder_MinMax(0,15);
			temp=irmp_data.command>>12;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if (temp==0)
				{
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(9,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			if (temp>0)
			{
				//hchstwertige Stelle <>0
				temp1=1;
			}
			else
			{
				temp1=0;
			}
			irmp_data.command&=0x0fff;
			irmp_data.command|=(temp<<12);
			
			//zweite Ziffer des Befehls
			lcd_gotoxy(10,1);
			encoder_MinMax(0,15);
			temp=(irmp_data.command>>8)&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if ((temp==0)&(temp1==0))
				{
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(10,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			if(temp>0) temp1=1;
			irmp_data.command&=0xf0ff;
			irmp_data.command|=(temp<<8);
			
			//dritte Ziffer des Befehls
			lcd_gotoxy(11,1);
			encoder_MinMax(0,15);
			temp=(irmp_data.command>>4)&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				if ((temp==0)&(temp1==0))
				{
					lcd_puts(" ");
				}
				else
				{
					lcd_puts(itoa(temp,s,16));
				}				
				lcd_gotoxy(11,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			irmp_data.command&=0xff0f;
			irmp_data.command|=(temp<<4);
			
			//vierte Ziffer des Befehls
			lcd_gotoxy(12,1);
			encoder_MinMax(0,15);
			temp=irmp_data.command&0x0f;
			encoder_write(temp);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				temp=encoder_read(1);
				lcd_puts(itoa(temp,s,16));
				lcd_gotoxy(12,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}
			irmp_data.command&=0xfff0;
			irmp_data.command|=temp;
			
			//Anzahl der Wiederholungen
			lcd_gotoxy(15,1);
			encoder_MinMax(0,14);
			encoder_write(irmp_data.flags);
			Button = encoder_GetButtonState();
			while(Button==ButtonPressed_Unpressed)
			{
				irmp_data.flags=encoder_read(1);
				lcd_puts(itoa(irmp_data.flags,s,16));
				lcd_gotoxy(15,1);
				_delay_ms(50);
				Button = encoder_GetButtonState();
			}

			
		}		
		
		if (Button==ButtonPressed_Long)
		{
			lcd_clrscr();
			if (irsnd_send_data(&irmp_data,TRUE))	//Telegramm senden und warten, bis gesendet
			{
				lcd_puts("Telegramm\ngesendet");
			}
			else
			{
				lcd_puts("Fehler beim\nSenden");
			}
		}
		else
		{
			lcd_clrscr();
			lcd_puts("abgebrochen");	
		}
				
		lcd_command(LCD_DISP_ON);	//Cursor off
		lcd_lightperm=0;
		_delay_ms(2000);
		lcd_clrscr();
	}

  }//End of Mainloop
}

